@@ -48,10 +48,9 @@ spawn_async = _glib.spawn_async
 
 
 def threads_init():
-    warnings.warn('threads_init no longer needs to be called. '
-                  'See: https://bugzilla.gnome.org/show_bug.cgi?id=686914',
-                  PyGIDeprecationWarning)
-
+    warnings.warn('Since version 3.10, calling threads_init is no longer needed. '
+                  'See: https://wiki.gnome.org/PyGObject/Threading',
+                  PyGIDeprecationWarning, stacklevel=2)
 
 __all__ += ['GError', 'OptionContext', 'OptionGroup', 'Pid',
             'spawn_async', 'threads_init']
