@@ -898,141 +898,12 @@ _pygi_argument_from_object (PyObject   *object,
     memset(&arg, 0, sizeof(GIArgument));
     type_tag = g_type_info_get_tag (type_info);
 
+    if (_pygi_marshal_from_py_basic_type (object, &arg, type_tag, transfer) ||
+            PyErr_Occurred()) {
+        return arg;
+    }
+
     switch (type_tag) {
-        case GI_TYPE_TAG_VOID:
-            g_warn_if_fail (transfer == GI_TRANSFER_NOTHING);
-            if (object == Py_None) {
-                arg.v_pointer = NULL;
-            } else if (!PYGLIB_PyLong_Check(object)  && !PyLong_Check(object)) {
-                PyErr_SetString(PyExc_TypeError,
-                    "Pointer assignment is restricted to integer values. "
-                    "See: https://bugzilla.gnome.org/show_bug.cgi?id=683599");
-            } else {
-                arg.v_pointer = PyLong_AsVoidPtr (object);
-            }
-            break;
-        case GI_TYPE_TAG_BOOLEAN:
-        {
-            arg.v_boolean = PyObject_IsTrue (object);
-            break;
-        }
-        case GI_TYPE_TAG_INT8:
-        case GI_TYPE_TAG_INT16:
-        case GI_TYPE_TAG_INT32:
-        {
-            PyObject *int_;
-
-            int_ = PYGLIB_PyNumber_Long (object);
-            if (int_ == NULL) {
-                PyErr_SetString (PyExc_TypeError, "expected int argument");
-                break;
-            }
-
-            if (type_tag == GI_TYPE_TAG_INT32)
-                arg.v_int32 = PYGLIB_PyLong_AsLong (int_);
-            else if (type_tag == GI_TYPE_TAG_INT8)
-                arg.v_int8 = PYGLIB_PyLong_AsLong (int_);
-            else if (type_tag == GI_TYPE_TAG_INT16)
-                arg.v_int16 = PYGLIB_PyLong_AsLong (int_);
-
-            Py_DECREF (int_);
-
-            break;
-        }
-        case GI_TYPE_TAG_UINT8:
-        case GI_TYPE_TAG_UINT16:
-        case GI_TYPE_TAG_UINT32:
-        case GI_TYPE_TAG_UINT64:
-        {
-            PyObject *number;
-            guint64 value;
-
-            number = PYGLIB_PyNumber_Long (object);
-            if (number == NULL) {
-                PyErr_SetString (PyExc_TypeError, "expected int argument");
-                break;
-            }
-
-#if PY_VERSION_HEX < 0x03000000
-            if (PyInt_Check (number)) {
-                value = PyInt_AS_LONG (number);
-            } else
-#endif
-            value = PyLong_AsUnsignedLongLong (number);
-
-            if (type_tag == GI_TYPE_TAG_UINT32)
-                arg.v_uint32 = value;
-            else if (type_tag == GI_TYPE_TAG_UINT64)
-                arg.v_uint64 = value;
-            else if (type_tag == GI_TYPE_TAG_UINT8)
-                arg.v_uint8 = value;
-            else if (type_tag == GI_TYPE_TAG_UINT16)
-                arg.v_uint16 = value;
-
-            Py_DECREF (number);
-
-            break;
-        }
-        case GI_TYPE_TAG_INT64:
-        {
-            PyObject *number;
-            gint64 value;
-
-            number = PYGLIB_PyNumber_Long (object);
-            if (number == NULL) {
-                PyErr_SetString (PyExc_TypeError, "expected int argument");
-                break;
-            }
-
-#if PY_VERSION_HEX < 0x03000000
-            if (PyInt_Check (number)) {
-                value = PyInt_AS_LONG (number);
-            } else
-#endif
-            value = PyLong_AsLongLong (number);
-
-            arg.v_int64 = value;
-
-            Py_DECREF (number);
-
-            break;
-        }
-        case GI_TYPE_TAG_FLOAT:
-        {
-            _pygi_marshal_from_py_float (NULL, NULL, NULL,
-                                         object, &arg);
-            break;
-        }
-        case GI_TYPE_TAG_DOUBLE:
-        {
-            _pygi_marshal_from_py_double (NULL, NULL, NULL,
-                                          object, &arg);
-            break;
-        }
-        case GI_TYPE_TAG_GTYPE:
-        {
-            arg.v_long = pyg_type_from_object (object);
-
-            break;
-        }
-        case GI_TYPE_TAG_UNICHAR:
-        {
-            _pygi_marshal_from_py_unichar (NULL, NULL, NULL,
-                                           object, &arg);
-            break;
-        }
-        case GI_TYPE_TAG_UTF8:
-        {
-            _pygi_marshal_from_py_utf8 (NULL, NULL, NULL,
-                                        object, &arg);
-            break;
-        }
-        case GI_TYPE_TAG_FILENAME:
-        {
-            _pygi_marshal_from_py_filename (NULL, NULL, NULL,
-                                            object, &arg);
-            break;
-        }
         case GI_TYPE_TAG_ARRAY:
         {
             Py_ssize_t length;
@@ -1156,16 +1027,16 @@ array_success:
                      * Further re-factoring is needed to fix this leak.
                      * See: https://bugzilla.gnome.org/show_bug.cgi?id=693405
                      */
-                    pygi_marshal_from_py_interface_struct (object,
-                                                           &arg,
-                                                           NULL, /*arg_name*/
-                                                           info, /*interface_info*/
-                                                           type_info,
-                                                           g_type,
-                                                           py_type,
-                                                           transfer,
-                                                           FALSE, /*is_caller_allocates*/
-                                                           g_struct_info_is_foreign (info));
+                    _pygi_marshal_from_py_interface_struct (object,
+                                                            &arg,
+                                                            NULL, /*arg_name*/
+                                                            info, /*interface_info*/
+                                                            type_info,
+                                                            g_type,
+                                                            py_type,
+                                                            transfer,
+                                                            FALSE, /*copy_reference*/
+                                                            g_struct_info_is_foreign (info));
 
                     Py_DECREF (py_type);
                     break;
@@ -1189,7 +1060,7 @@ array_success:
                 case GI_INFO_TYPE_INTERFACE:
                 case GI_INFO_TYPE_OBJECT:
                     /* An error within this call will result in a NULL arg */
-                    pygi_marshal_from_py_gobject (object, &arg, transfer);
+                    _pygi_marshal_from_py_gobject_out_arg (object, &arg, transfer);
                     break;
 
                 default:
@@ -1373,6 +1244,8 @@ hash_table_release:
             PyErr_SetString (PyExc_NotImplementedError, "error marshalling is not supported yet");
             /* TODO */
             break;
+        default:
+            g_assert_not_reached ();
     }
 
     return arg;
@@ -1380,14 +1253,14 @@ hash_table_release:
 
 /**
  * _pygi_argument_to_object:
- * @arg: The argument to convert to an object. 
+ * @arg: The argument to convert to an object.
  * @type_info: Type info for @arg
  * @transfer:
  *
  * If the argument is of type array, it must be encoded in a GArray, by calling
  * _pygi_argument_to_array(). This logic can not be folded into this method
  * as determining array lengths may require access to method call arguments.
- * 
+ *
  * Returns: A PyObject representing @arg
  */
 PyObject *
@@ -1399,6 +1272,10 @@ _pygi_argument_to_object (GIArgument  *arg,
     PyObject *object = NULL;
 
     type_tag = g_type_info_get_tag (type_info);
+    object = _pygi_marshal_to_py_basic_type (arg, type_tag, transfer);
+    if (object)
+        return object;
+
     switch (type_tag) {
         case GI_TYPE_TAG_VOID:
         {
@@ -1408,84 +1285,6 @@ _pygi_argument_to_object (GIArgument  *arg,
             }
             break;
         }
-        case GI_TYPE_TAG_BOOLEAN:
-        {
-            object = PyBool_FromLong (arg->v_boolean);
-            break;
-        }
-        case GI_TYPE_TAG_INT8:
-        {
-            object = PYGLIB_PyLong_FromLong (arg->v_int8);
-            break;
-        }
-        case GI_TYPE_TAG_UINT8:
-        {
-            object = PYGLIB_PyLong_FromLong (arg->v_uint8);
-            break;
-        }
-        case GI_TYPE_TAG_INT16:
-        {
-            object = PYGLIB_PyLong_FromLong (arg->v_int16);
-            break;
-        }
-        case GI_TYPE_TAG_UINT16:
-        {
-            object = PYGLIB_PyLong_FromLong (arg->v_uint16);
-            break;
-        }
-        case GI_TYPE_TAG_INT32:
-        {
-            object = PYGLIB_PyLong_FromLong (arg->v_int32);
-            break;
-        }
-        case GI_TYPE_TAG_UINT32:
-        {
-            object = PyLong_FromLongLong (arg->v_uint32);
-            break;
-        }
-        case GI_TYPE_TAG_INT64:
-        {
-            object = PyLong_FromLongLong (arg->v_int64);
-            break;
-        }
-        case GI_TYPE_TAG_UINT64:
-        {
-            object = PyLong_FromUnsignedLongLong (arg->v_uint64);
-            break;
-        }
-        case GI_TYPE_TAG_FLOAT:
-        {
-            object = PyFloat_FromDouble (arg->v_float);
-            break;
-        }
-        case GI_TYPE_TAG_DOUBLE:
-        {
-            object = PyFloat_FromDouble (arg->v_double);
-            break;
-        }
-        case GI_TYPE_TAG_GTYPE:
-        {
-            object = pyg_type_wrapper_new ( (GType) arg->v_long);
-            break;
-        }
-        case GI_TYPE_TAG_UNICHAR:
-        {
-            object = _pygi_marshal_to_py_unichar (NULL, NULL, NULL,
-                                                  arg);
-            break;
-        }
-        case GI_TYPE_TAG_UTF8:
-        {
-            object = _pygi_marshal_to_py_utf8 (NULL, NULL, NULL,
-                                               arg);
-            break;
-        }
-        case GI_TYPE_TAG_FILENAME:
-        {
-            object = _pygi_marshal_to_py_filename (NULL, NULL, NULL,
-                                                   arg);
-            break;
-        }
         case GI_TYPE_TAG_ARRAY:
         {
             /* Arrays are assumed to be packed in a GArray */
@@ -1571,13 +1370,13 @@ _pygi_argument_to_object (GIArgument  *arg,
                         py_type = _pygi_type_get_from_g_type (g_type);
                     }
 
-                    object = pygi_marshal_to_py_interface_struct (arg,
-                                                                  info, /*interface_info*/
-                                                                  g_type,
-                                                                  py_type,
-                                                                  transfer,
-                                                                  FALSE, /*is_allocated*/
-                                                                  g_struct_info_is_foreign (info));
+                    object = _pygi_marshal_to_py_interface_struct (arg,
+                                                                   info, /*interface_info*/
+                                                                   g_type,
+                                                                   py_type,
+                                                                   transfer,
+                                                                   FALSE, /*is_allocated*/
+                                                                   g_struct_info_is_foreign (info));
 
                     Py_XDECREF (py_type);
                     break;
@@ -1634,10 +1433,10 @@ _pygi_argument_to_object (GIArgument  *arg,
                             transfer == GI_TRANSFER_NOTHING &&
                             g_object_is_floating (arg->v_pointer)) {
                         g_object_ref (arg->v_pointer);
-                        object = pygi_marshal_to_py_object (arg, GI_TRANSFER_EVERYTHING);
+                        object = _pygi_marshal_to_py_object (arg, GI_TRANSFER_EVERYTHING);
                         g_object_force_floating (arg->v_pointer);
                     } else {
-                        object = pygi_marshal_to_py_object (arg, transfer);
+                        object = _pygi_marshal_to_py_object (arg, transfer);
                     }
 
                     break;
@@ -1775,6 +1574,11 @@ _pygi_argument_to_object (GIArgument  *arg,
                 Py_INCREF (object);
                 break;
             }
+            break;
+        }
+        default:
+        {
+            g_assert_not_reached();
         }
     }
 
@@ -1899,8 +1703,8 @@ _pygi_argument_from_g_value(const GValue *value,
             arg.v_pointer = g_value_get_boxed (value);
             break;
         case GI_TYPE_TAG_VOID:
-            g_critical("Converting of type '%s' is not implemented", g_type_tag_to_string(type_tag));
-            g_assert_not_reached();
+            arg.v_pointer = g_value_get_pointer (value);
+            break;
     }
 
     return arg;
