@@ -26,11 +26,11 @@
 
 #if PY_VERSION_HEX < 0x03000000
 #include <pycairo.h>
+static Pycairo_CAPI_t *Pycairo_CAPI;
 #else
 #include <pycairo/py3cairo.h>
 #endif
 
-static Pycairo_CAPI_t *Pycairo_CAPI;
 
 #include "pygi-foreign.h"
 
@@ -191,7 +191,7 @@ PYGLIB_MODULE_START(_gi_cairo, "_gi_cairo")
 #if PY_VERSION_HEX < 0x03000000
     Pycairo_IMPORT;
 #else
-    Pycairo_CAPI = (Pycairo_CAPI_t*) PyCapsule_Import("cairo.CAPI", 0);
+    import_cairo();
 #endif
 
     if (Pycairo_CAPI == NULL)
